/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/*=============================================================================
   ToDo:

=============================================================================*/

/*=============================================================================
   Note:

=============================================================================*/

#include <STDriveOps.h>

STDriveOps::STDriveOps()
: STObjectController()
{
   ParserOps1 = new STParserOps();
   Time1 = new STTime();

   DefaultPermissions = 777;
   ResetSearchPeramiters();
}

STDriveOps::~STDriveOps()
{
   delete ParserOps1;
   delete Time1;
}

void STDriveOps::ResetSearchPeramiters()
{
   SearchPeramiters.FileAndPathMatch = "";
   
   SearchPeramiters.FileAndPathAvoid = "";
   SearchPeramiters.FileMatch = "";
   SearchPeramiters.FileAvoid = "";
   SearchPeramiters.FileExtentionMatch = "";
   SearchPeramiters.FileExtentionAvoid = "";
   SearchPeramiters.PathMatch = "";
   SearchPeramiters.PathAvoid = "";
   SearchPeramiters.DriveMatch = "";
   SearchPeramiters.DriveAvoid = "";
                  
   SearchPeramiters.FileSizeMin = 0;
   SearchPeramiters.FileSizeMax = 0;
   SearchPeramiters.FileMinDate = 0;
   SearchPeramiters.FileMaxDate = 0;
   SearchPeramiters.FileAttributesMatch = 0;
   SearchPeramiters.FileAttributesAvoid = 0;

   // Many of these to not apply in Linux but I kept them
   // to be compatible with my Delphi app so I can port them.
   ReturnParts.IncludeDrive = false;
   ReturnParts.IncludePath = false;
   ReturnParts.IncludeFileName = false;
   ReturnParts.IncludeFileExtention = false;
   ReturnParts.IncludeFileSize = false;
   ReturnParts.IncludeFileDate = false;
   ReturnParts.IncludeFileAttibutes = false;
   ReturnParts.IncludePathSize = false;
   ReturnParts.IncludePathDate = false;
   ReturnParts.IncludeDirectoryAttributes = false;
   ReturnParts.IncludeFileAttributes = false;     

}

double STDriveOps::GetFileMDate(const char *PathFileName)
{
   QString TempString;
   
   TempString = PathFileName;
   return(GetFileMDate(TempString));
}

double STDriveOps::GetFileMDate(QString &PathFileName)
{
   double TempReturn;
   struct stat StatPointer; 	// Pointer to entery stats
   STTime TimeConvert;
   
   TempReturn = 0;
   
   if(FileExists(PathFileName) == true)
   {
      if(stat(PathFileName, &StatPointer)==0)   
      {
         TempReturn = TimeConvert.TimeInFracOfDay(StatPointer.st_mtime);
         TempReturn += TimeConvert.DaysCount(StatPointer.st_mtime);
      }
   }

   return(TempReturn);
}

QString STDriveOps::ExtractFileName(const char *PathFileName)
{
   QString TheCharSting;
   
   TheCharSting = PathFileName;
   ReturnString = ExtractFileName(TheCharSting);
   return(TheCharSting);
}

QString STDriveOps::ExtractFileName(QString &PathFileName)
{
   QString ParsingString;
   STStringsEx Stringx;
   
   ParsingString = PathFileName;
   if(ParsingString.find("/",0,true) > -1)
   {
      ParsingString.remove(0,Stringx.LastPos("/",ParsingString)+1);
   }
   ReturnString = ParsingString;
   return(ReturnString);   
}

QString STDriveOps::ExtractPath(const char *ProvidedPathFile)
{
   QString TempString;
   
   TempString = ProvidedPathFile;
   ReturnString = ExtractPath(TempString);
   return(ReturnString);
}

QString STDriveOps::ExtractPath(QString &ProvidedPathFile)
{
   QString ParsingString;
   STStringsEx Stringx;
   
   ParsingString = ProvidedPathFile;
   if(ParsingString.find("/",0,false) > -1)
   {
      ParsingString.remove(Stringx.LastPos("/",ParsingString),ParsingString.length()+1);
   }
   ReturnString = ParsingString;
   return(ReturnString);   
}

bool STDriveOps::FileExists(const char *PathFileName)
{
   QString TheStringToTest;
   
   TheStringToTest = PathFileName;
   return(FileExists(TheStringToTest));
}

bool STDriveOps::FileExists(QString &PathFileName)
{
   struct stat StatPointer; 	// Pointer to entery stats
   bool TempReturn;

   TempReturn = false;
   if(stat(PathFileName, &StatPointer)==0)   
   {
      if(S_ISREG(StatPointer.st_mode))
      {
         TempReturn = true;
      }
   }   	
   return(TempReturn);
}

bool STDriveOps::DirExists(const char *PathFileName)
{
   QString TheStringToTest;
   
   TheStringToTest = PathFileName;
   return(DirExists(TheStringToTest));
}

bool STDriveOps::DirExists(QString &PathFileName)
{
   struct stat StatPointer; 	// Pointer to entery stats
   bool TempReturn;

   TempReturn = false;
   if(stat(PathFileName, &StatPointer)==0)   
   {
      if(S_ISDIR(StatPointer.st_mode))
      {
         TempReturn = true;
      }
   }   	
   return(TempReturn);
}

bool STDriveOps::MakePath(const char *PathToCreate)
{
   QString TempPathName;
   
   TempPathName = PathToCreate;
   return(MakePath(TempPathName));
}

bool STDriveOps::MakePath(QString &PathToCreate)
{
   QString NewSub,PathToMake,FullPath;
   bool Sucessfull;
   int Permissions, OwnerPermissions, GroupPermissions, OtherPermissions, CarryOver;
   
   OtherPermissions = DefaultPermissions % 10;
   CarryOver = (int)DefaultPermissions / 10;
   
   GroupPermissions = CarryOver % 10;
   CarryOver = (int)CarryOver / 10;
    
   OwnerPermissions = CarryOver;
    
   Permissions = OwnerPermissions;
   Permissions <<= 4;
   Permissions += GroupPermissions;
   Permissions <<= 4;
   Permissions += OtherPermissions;
   
   Sucessfull = true;
   PathToMake = PathToCreate;
   if(PathToMake[PathToMake.length()-1] != '/')
   {
      PathToMake += "/";
   }
   FullPath = "";
   while(PathToMake > "")
   {
      NewSub = PathToMake;
      if(NewSub.find("/",0,true) > -1)
      {
         NewSub.remove(NewSub.find("/",0,true),NewSub.length());
         PathToMake.remove(0,PathToMake.find("/",0,true)+1);
      }
      FullPath += "/"+NewSub;
      while(FullPath.find("//",0,false) > -1)
      {
         FullPath.remove(FullPath.find("//",0,false),1);
      }
      if(FullPath.length() > 200)
      {
         printf("STDriveOps::MakePath: Warning posible infinite recursion loop!\n");
         printf("New path creation is delayed by 5 seconds.\n");
         sleep(5);
      }
      if(DebugLevel > 2)
      {
         printf(" -MakePath mkdir = %s\n",(const char *)FullPath);
         printf("  with permissions of %d",Permissions);
      }
      if((FileExists(FullPath) == false)&&(DirExists(FullPath) == false))
      {
         mkdir(FullPath,511); //Permissions);
      } else
      {         
         if(FileExists(FullPath) == true)
         {
            Sucessfull = false;
            break;
         }
      }
   }
   return(Sucessfull);
}

QString STDriveOps::GetDir()
{
   unsigned int PathNameSize;
   bool BufferIsLargeEnough;
   int BufferIncroments;
   char *BufferPointer;
   
   PathNameSize = 0;
   BufferPointer = NULL;
   BufferIncroments = 1024;
   ReturnString = "";
   BufferIsLargeEnough = false;
   while(BufferIsLargeEnough == false)
   {
      if(BufferPointer != NULL)
      {
         free(BufferPointer);
         BufferPointer = NULL;
      }
      PathNameSize += BufferIncroments;
      BufferPointer = (char *)malloc(PathNameSize);
      getcwd(BufferPointer,PathNameSize);
      
      if(strlen(BufferPointer) < PathNameSize - 2)
      {
         BufferIsLargeEnough = true;
      }
   }
   ReturnString = BufferPointer;
   
   if(ReturnString > "") 
   {
      if(ReturnString[ReturnString.length()-1] != '/')
      {
         ReturnString+="/";
      }
   }

   free(BufferPointer);
   return(ReturnString);   
}

void STDriveOps::RecurseFiles(STStringList *FilesStringList, int LevelOfRecurtion,
   QString &DirectoryToStartIn)
{
   QString OrininalDir;
   
   FilesStringList->Clear();
   OrininalDir = GetDir();
   if((DirectoryToStartIn > "")&&(DirExists(DirectoryToStartIn) == true))
   {
      chdir(DirectoryToStartIn);
      RecurseFiles(FilesStringList,LevelOfRecurtion);
      chdir(OrininalDir);
   }
}

void STDriveOps::RecurseFiles(STStringList *FilesStringList, int LevelOfRecurtion,
   char *DirectoryToStartIn)
{
   QString DirToStartInString;
   
   DirToStartInString = DirectoryToStartIn;
   RecurseFiles(FilesStringList,LevelOfRecurtion,DirToStartInString);
}

void STDriveOps::RecurseFiles(STStringList *FilesStringList, int LevelOfRecurtion)
{
   STStringList *DirStringList, *AppendFilesList;
   QString CurrentDirName,AppendString,PathToFile;
   QString FullPathAndFile, SizeString;
  
   CurrentDirName = GetDir();
   
   DirStringList = new STStringList();
   AppendFilesList = new STStringList();
   FilesStringList->Clear();
   
   if(DebugLevel > 2)
   {
      printf("STDriveOps::RecurseFiles\n");
   }
   if(LevelOfRecurtion != 1)
   {
      RecursePath(DirStringList,LevelOfRecurtion-1);
   } else
   {
      DirStringList->Add(CurrentDirName);
   }
   

   //SearchPeramiters.FileAttributesAvoid.Directory :=True;
   DirStringList->Add(CurrentDirName);

   if(DebugLevel > 2)
   {
      printf("DirStringList->Count() = %d\n",DirStringList->Count());
   }
   
   while(DirStringList->Count() > 0) 
   {
      PathToFile = DirStringList->Item(0);
      chdir(PathToFile);
      GetFiles(AppendFilesList);

      if(DebugLevel > 2)
      {
         printf("AppendFilesList->Count() = %d\n",AppendFilesList->Count());
      }
      
      while(AppendFilesList->Count() > 0)
      {
         FullPathAndFile = PathToFile;
         FullPathAndFile += AppendFilesList->Item(0);
         AppendString = "";
         if(ReturnParts.IncludePath == true)
         {
            AppendString = PathToFile;
         }
         
         AppendString += AppendFilesList->Item(0);
                  
         if(ReturnParts.IncludeFileSize == true)
         {
            SizeString.sprintf("%d",FileSize(FullPathAndFile));
            AppendString +=":"+SizeString;            
         }
         if(ReturnParts.IncludeFileDate == true)
         {
            AppendString +=":"+Time1->DateTime2String(GetFileMDate(FullPathAndFile),0);            
         }                  
         FilesStringList->Add(AppendString);
         AppendFilesList->Delete(0);
      }
      DirStringList->Delete(0);            
   }
   chdir(CurrentDirName);
   delete DirStringList;
   delete AppendFilesList;
}

QString STDriveOps::GetParentDir(QString &ChildPath)
{
   QString EditingString,UpSearchPath;
   int PresumedParent;
   bool DoneUpSearching;
   
   // Absolute path
   if(EditingString.find("/",0,true) == 0)
   {      
      if(EditingString[EditingString.length()-1] == '/')
      {
         EditingString.remove(EditingString.length()-1,1);
      }
      if(EditingString.length() > 0)
      {
         EditingString.remove(EditingString.find("/",-1,true),EditingString.length());         
      }
      if(EditingString.length() == 0)
      {
         EditingString = "/";
      }   
      ReturnString = EditingString;
   } else   
   // If the ChildPath starts with ../
   if(EditingString.find("../",0,true) == 0)
   {
      PresumedParent = 0;
      EditingString = ChildPath;
      while(EditingString.find("../",0,true) == 0)
      {
         PresumedParent++;
      }
      UpSearchPath = GetDir();
      DoneUpSearching = false;
      while(DoneUpSearching == false)
      {
         if((UpSearchPath.length() > 0)&&(PresumedParent > 0))
         {
            UpSearchPath.remove(UpSearchPath.find("/",-1,true),UpSearchPath.length());
            PresumedParent--;
         } else
         {
            DoneUpSearching = true;
         }
      }
      ReturnString = UpSearchPath;
   } else
   // If the ChildPath starts with ./
   if(EditingString.find("./",0,true) == 0)
   {
      EditingString = GetDir();
      if(EditingString[EditingString.length()-1] == '/')
      {
         EditingString.remove(EditingString.length()-1,1);
      }
      if(EditingString.find("/",0,true) > 0)
      {
         EditingString.remove(EditingString.find("/",0,true),EditingString.length());
      }
      ReturnString = EditingString;
   } else
   // Assumed the path is a sub of the current, the parent of the sub would be the current.
   {
      ReturnString = GetDir();
   }
   return(ReturnString);
}

void STDriveOps::GetDirectories(STStringList *DirectoryList)
{
   QString CurrentDir;
   
   CurrentDir = GetDir();
   GetDirectories(DirectoryList,CurrentDir);   
}

void STDriveOps::GetDirectories(STStringList *DirectoryList, QString &StartingDir)
{
   DIR *DirPointer;  			// Poniter to directory informoation.
   struct dirent *DirEnt; 		// Structure of directory enteries
   struct stat StatPointer; 	// Pointer to entery stats
   QString DirectoryName;
   QString StartHereDir;
   QString DirPrefix,FullEnteryName;
   int StatError;

   DirectoryList->Clear();
   
   StartHereDir = StartingDir;
      
   DirPrefix = StartHereDir;
   if(StartHereDir.find("./",0,true) != 0)
   {
      while(StartHereDir.find("../",0,true) == 0)
      {
         DirPrefix = GetParentDir(DirPrefix);
         StartHereDir.remove(0,3);
      }
   }
   //printf("StartHereDir=%s\n",(const char *)StartHereDir);
   //chdir(StartHereDir);
   DirPointer = opendir(StartHereDir);   
   DirEnt = readdir(DirPointer);

   while(DirEnt != NULL)
   {      
      if((strcmp(DirEnt->d_name, ".") != 0) && (strcmp(DirEnt->d_name, "..") != 0))
      {
         FullEnteryName = DirPrefix+DirEnt->d_name;
         //printf("FullEnteryName=%s\n",(const char *)FullEnteryName);
         StatError = stat(FullEnteryName, &StatPointer);
         /*switch(StatError)
         {
            case EACCES: printf("Permission denied.\n"); break;
            case EMFILE: printf("Too many file descriptors in use by process.\n"); break;
            case ENFILE: printf("Too many files are currently open in the system.\n"); break;
            case ENOENT: printf("Directory  does  not  exist, or %s is an empty string.\n",
               (const char *)StartHereDir); break;
            case ENOMEM: printf("Insufficient memory to complete the operation.\n"); break;
            case ENOTDIR: printf("Error, %s is not a directory.\n",(const char *)StartHereDir); break;         
            default: break;
         }*/
         if(StatError==0)
         {
            if(S_ISDIR(StatPointer.st_mode))
            {
               DirectoryName = DirEnt->d_name;
               if(DirectoryName[DirectoryName.length()-1] != '/')
               {
                  DirectoryName+="/";
               }
               if(FullEnteryName.find(DirPrefix,0,true) > -1)
               {
                  FullEnteryName.remove(0,DirPrefix.length());
               }
               //printf("adding %s\n",(const char *)FullEnteryName);
               
               DirectoryList->Add(DirectoryName);
            }
         }	  
      }
      DirEnt = readdir(DirPointer);
   }   	
   closedir(DirPointer);
}

int STDriveOps::KillDir(QString &DirToDelete)
{
   QString OringinalDir;
   unsigned int ALoopCounter;
   int TempReturn;
   STStringList TheKillList;
   QString KillThisObject;
   
   OringinalDir = GetDir();
   TempReturn = 0;
   
   // Empty the dir of all files and files in sub directories.
   ResetSearchPeramiters();
   ReturnParts.IncludePath = true;
   RecurseFiles(&TheKillList,0,DirToDelete);
   for(ALoopCounter=0;ALoopCounter<TheKillList.Count();ALoopCounter++)
   {
      KillThisObject = TheKillList.Item(ALoopCounter);
      TempReturn = KillFile(KillThisObject);
      if(TempReturn != 0)
      {
         printf("STDriveOps::KillDir: could not delete %s\n",(const char *)KillThisObject);
         break;
      }
   }
   
   // Empty the dir of all sub directories.
   if(TempReturn == 0)
   {
      TheKillList.Clear();
      ResetSearchPeramiters();
      ReturnParts.IncludePath = true;
      RecursePath(&TheKillList,0,DirToDelete);
      for(ALoopCounter=TheKillList.Count();ALoopCounter>0;ALoopCounter--)
      {
         KillThisObject = TheKillList.Item(ALoopCounter-1);
         TempReturn = rmdir(KillThisObject);
         if(TempReturn != 0)
         {
            printf("STDriveOps::KillDir: could not delete %s\n",(const char *)KillThisObject);            
            break;
         }
      }   
   }

   // Finaly remove the dir that we were asked to.
   if(TempReturn == 0)
   {   
      TempReturn = rmdir(DirToDelete);
      if(TempReturn != 0)
      {
         printf("STDriveOps::KillDir: could not delete %s\n",(const char *)KillThisObject);
      }
   }
   
   return(TempReturn);
}

int STDriveOps::KillDir(const char *DirToDelete)
{
   QString DirString;
   
   DirString = DirToDelete;
   return(KillDir(DirString));   
}

int STDriveOps::KillFile(QString &FileToDelete)
{
   char *TempChars;
   int TempReturn;
   
   TempChars = (char *)malloc(FileToDelete.length()+1);
   strcpy(TempChars,FileToDelete);
   TempChars[FileToDelete.length()]='\0';   
   TempReturn = KillFile(TempChars);
   free(TempChars);
   
   return(TempReturn);   
}

void STDriveOps::FileIOErrorCodes()
{
   switch(errno)
   {
      case EFAULT: printf("Error: Path name points out side your accessible address space.\n"); break;         
      case EACCES: printf("Error: Permission denied.\n"); break;         
      case EPERM: printf("Error: Sticky bit is set.\n"); break;
      case ENAMETOOLONG: printf("Error: Path name is to long.\n"); break;
      case EROFS: printf("Error: Read-only file system.\n"); break;
      case EMFILE: printf("Error: Too many file descriptors in use by process.\n"); break;
      case ENFILE: printf("Error: Too many files are currently open in the system.\n"); break;
      case ENOENT: printf("Error: Directory entry does not exist, or string is empty.\n"); break;
      case ENOMEM: printf("Error: Insufficient memory to complete the operation.\n"); break;
      case ENOTDIR: printf("Error: Give string is not a directory.\n"); break;         
      default: printf("Error: Code %d\n",errno); break;
   }
}

int STDriveOps::KillFile(const char *FileToDelete)
{
   int ReturnError;

   if(DebugLevel > 2)
   {
      printf("remove %s\n",FileToDelete);
   }
   ReturnError = remove(FileToDelete);
   if(ReturnError != 0)
   {
      FileIOErrorCodes();
   }
   if(DebugLevel > 1)
   {
      printf("ReturnError = %d\n",ReturnError);
   }
   return(ReturnError);
}

void STDriveOps::GetFiles(STStringList *FilesList)
{
   QString CurrentDir;
   
   CurrentDir = GetDir();
   GetFiles(FilesList,CurrentDir);   
}

void STDriveOps::GetFiles(STStringList *FilesList, QString &StartingDir)
{
   DIR *DirPointer;  			// Poniter to directory informoation.
   struct dirent *DirEnt; 		// Structure of directory enteries
   struct stat StatPointer; 	// Pointer to entery stats
   QString FileName;
   bool FileIsQualified;
   QString DirPrefix,FullEnteryName;
   QString StartHereDir;
   int StatError;

   FilesList->Clear();

   ParserOps1->SetDebugLevel(DebugLevel);
   
   StartHereDir = StartingDir;
   
   if(StartHereDir[StartHereDir.length()-1] != '\0')
   {
      StartHereDir += "\0";
   }
   
   DirPrefix = StartHereDir;
   if(StartHereDir.find("./",0,true) > -1)
   {
      DirPrefix = "";
      while(StartHereDir.find("../",0,true) == 0)
      {
         DirPrefix = GetParentDir(DirPrefix);
         StartHereDir.remove(0,3);
      }
   }

   DirPointer = opendir(StartHereDir);   
   DirEnt = readdir(DirPointer);

   while(DirEnt != NULL)
   {
      if((strcmp(DirEnt->d_name, ".") != 0) && (strcmp(DirEnt->d_name, "..") != 0))
      {
         FullEnteryName = DirPrefix+DirEnt->d_name;
         StatError = stat(FullEnteryName, &StatPointer);
         if(DebugLevel > 2)
         {
            printf("FullEnteryName = %s\n",(const char *)FullEnteryName);
         }
         if(StatError != 0)
         {
            FileIOErrorCodes();
         }
         if(StatError==0)
         {
            if(S_ISREG(StatPointer.st_mode))
            {
               FileName = DirEnt->d_name;
               FileIsQualified = true; 
               if(SearchPeramiters.FileMatch != "")
               {
                  FileIsQualified = false;
                  if(DebugLevel > 2)
                  {
                     printf("WildCardCompairStrings = %s -> %s\n",
                     (const char *)SearchPeramiters.FileMatch,(const char*)FileName);
                  }
                                    
                  if(ParserOps1->WildCardCompairStrings(SearchPeramiters.FileMatch,FileName,false) == true)
                  {
                     if(DebugLevel > 2)
                     {
                        printf("Match = %s -> %s\n",
                          (const char *)SearchPeramiters.FileMatch,(const char*)FileName);
                     }
                     FileIsQualified = true;
                  }
               }
               
               if(FileIsQualified == true)
               {
                  FilesList->Add(FileName);               
               }
            }
         }	  
      }
      DirEnt = readdir(DirPointer);
   }   	
   closedir(DirPointer);
}

void STDriveOps::RecursePath(STStringList *PathStringList, int LevelOfRecurtion,
   QString &DirectoryToStartIn)
{
   QString OrininalDir;
   
   PathStringList->Clear();
   OrininalDir = GetDir();
   if((DirectoryToStartIn > "")&&(DirExists(DirectoryToStartIn) == true))
   {
      chdir(DirectoryToStartIn);
      RecursePath(PathStringList,LevelOfRecurtion);
      chdir(OrininalDir);
   }
}

void STDriveOps::RecursePath(STStringList *PathStringList, int LevelOfRecurtion,
   char *DirectoryToStartIn)
{
   QString DirToStartInString;
   
   DirToStartInString = DirectoryToStartIn;
   RecursePath(PathStringList,LevelOfRecurtion,DirToStartInString);
}

void STDriveOps::RecursePath(STStringList *PathStringList, int LevelOfRecurtion)
{
   QString CurrentDirName;
   QString ASubDirName;
   unsigned int ALoopCounter; //, ErrorCode;
   STStringList *SubDirectories;

   CurrentDirName = GetDir();

   // Directory Enteries
   SubDirectories = new STStringList();
   GetDirectories(SubDirectories);
   for(ALoopCounter = 0;ALoopCounter < SubDirectories->Count();ALoopCounter++)
   {
      ASubDirName = SubDirectories->Item(ALoopCounter);
      PathStringList->Add(CurrentDirName+ASubDirName);
      if(LevelOfRecurtion != 1)
      {
         chdir(ASubDirName);
         RecursePath(PathStringList,LevelOfRecurtion-1);
         chdir(CurrentDirName);
      }
   }  
   chdir(CurrentDirName);
   delete SubDirectories;
}

int STDriveOps::FileSize(QString &FilePathName)
{
   int ReturnValue, SucessTest;
   struct stat StatusBuffer;

   if(DebugLevel > 2)
   {
      printf("STDriveOps::FileSize(%s)\n",(const char *)FilePathName);
   }
   
   ReturnValue = -1;
   SucessTest = lstat(FilePathName,&StatusBuffer);
   if(SucessTest > -1)
   {
      ReturnValue = StatusBuffer.st_size;
   }  

   if(DebugLevel > 3)
   {
      printf("ReturnValue = %d\n",ReturnValue);
   }
   
   return(ReturnValue);
}


