/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/*=============================================================================
   ToDo:

=============================================================================*/

/*=============================================================================
   Note: Messages will be in this format
   
   - Messages will be created as text files in /tmp/<App name>_<App pid>
      where <App name> is the name of the app with no spaces and <App pid> is
      the PID of the app.
   - Contents of the message file will be 
      :<From <App name>_<App pid>>:<Message>:
      where the first char of the file will be a marker char to seperate fields
      and is shown here as a colen and for this version of the object will be a :,
      the first filed is the from to tell what app send the message and to what app
      a reply message can be sent to. The second filed is the message it self and
      the message is ended by a marker char, in this case a :
      
      

=============================================================================*/

#include <STDynamicDataLink.h>

STDynamicDataLink::STDynamicDataLink() 
{
   SystemAPI1 = new STSystemAPI();
   MessageList = new STStringList();
   
   CreateMyMessageFile();
}


STDynamicDataLink::~STDynamicDataLink()
{
   QString FlushCommand;
   QString MyFileName;
   
   MyFileName = MyMessageFile();

   // Flush message file into a temp file we can read and delete
   FlushCommand = "rm -f "+MyFileName;
   system(FlushCommand);

   delete SystemAPI1;
   delete MessageList;
}

QString STDynamicDataLink::MessageFileName(int UseThisPid)
{
   QString NumberString;

   ReturnString = "/tmp/"+SystemAPI1->GetAppNameFromPid(UseThisPid);
   ReturnString += "_";
   NumberString.sprintf("%d",UseThisPid);
   ReturnString += NumberString;
   
   return(ReturnString);
}

QString STDynamicDataLink::MyMessageFile()
{
   int MyPidNumber;
   
   MyPidNumber = SystemAPI1->GetAppPid();
   ReturnString = MessageFileName(MyPidNumber);
   return(ReturnString);
}

bool STDynamicDataLink::IsNewMessageWaiting()
{
   QString FileName;
   //int OpenFileIndex;
   QString TheMessage;
   bool TempReturn;
   STDriveOps DriveOps1;

   TempReturn = false;
   FileName = MyMessageFile();
   
   // Any valid message will have 3 marker chars plus data
   if(DriveOps1.FileSize(FileName) > 3)
   {
      TempReturn = true;
   }
   return(TempReturn);
}

bool STDynamicDataLink::IsMessageWaiting()
{
   bool TempReturn;

   TempReturn = false;
   if(IsNewMessageWaiting() == true)
   {
      TempReturn = true;
   }
   if(MessageList->Count() > 0)
   {
      TempReturn = true;
   }   
   return(TempReturn);
}

void STDynamicDataLink::CreateMyMessageFile()
{
   int FilePointer;
   QString ChmodInstruction;
   QString FileName;
      
   // Recreate my file
   FileName = MyMessageFile();
   FilePointer = open(FileName, O_WRONLY | O_CREAT);
   close(FilePointer);   
   ChmodInstruction = "chmod 666 "+FileName;
   system(ChmodInstruction);
}

void STDynamicDataLink::ReplaceMessage(QString &OldMessage)
{
   MessageList->Add(OldMessage);
}

QString STDynamicDataLink::ReadMessage()
{
   STStringList *TempMessageList;
   QString FlushCommand;
   QString MyFileName;
   unsigned int ALoopCounter;
   
   ReturnString = "";
   if(IsNewMessageWaiting() == true)
   {
      MyFileName = MyMessageFile();

      // Flush message file into a temp file we can read and delete
      FlushCommand = "mv -f "+MyFileName;
      FlushCommand += " "+MyFileName+"_X";      
      system(FlushCommand);
   
      CreateMyMessageFile();

      FlushCommand = MyFileName+"_X";
   
      TempMessageList = new STStringList();
      TempMessageList->LoadFromFile(FlushCommand);
      for(ALoopCounter=0;ALoopCounter < TempMessageList->Count();ALoopCounter++)
      {
         MessageList->Add(TempMessageList->Item(ALoopCounter));   
      }   
      delete TempMessageList;
      
      // Done with temp file, flush it
      FlushCommand = "rm -f "+MyFileName+"_X";
      system(FlushCommand);
   
      ReturnString = MessageList->Item(0);
      MessageList->Remove(0);      
   } else
   if(MessageList->Count() > 0)
   {
      ReturnString = MessageList->Item(0);
      MessageList->Remove(0);         
   }
   
   return(ReturnString);
}

QString STDynamicDataLink::ExtractSender(QString &OriginalMessage)
{
   QString Part1;
   
   ReturnString = "";
   
   Part1 = OriginalMessage;      
   Part1.remove(0,Part1.find(":",0,false)+1);
   Part1.remove(Part1.find(":",0,false),Part1.length());      
   ReturnString = Part1;
   
   return(ReturnString);
}

QString STDynamicDataLink::ExtractMessage(QString &OriginalMessage)
{
   QString Part1;
   
   ReturnString = "";
   
   Part1 = OriginalMessage;      
   Part1.remove(0,Part1.find(":",0,false)+1);
   Part1.remove(0,Part1.find(":",0,false)+1);
   Part1.remove(Part1.find(":",0,false),Part1.length());      
   ReturnString = Part1;
   
   return(ReturnString);
}

bool STDynamicDataLink::SendMessage(QString &TheMessage, int PidToSendTo, QString &AppNameToSendTo)
{
   bool SucessfullySend;
   STIntList *PidList;
   QString FileName;
   int FilePointer;
   QString FormedMessage;
      
   SucessfullySend = false;
  
   // If we are not given the PID we can use the app name but there can only be
   // one running app by that name for it to work.
   if(PidToSendTo == 0)
   {
      if(AppNameToSendTo != "")
      {
         PidList = new STIntList();
         SystemAPI1->GetPidList(AppNameToSendTo,PidList,true);
         if(PidList->Count() == 1)
         {
            PidToSendTo = PidList->Item(0);
         }
         delete PidList;
      }
   }
   
   if(PidToSendTo != 0)
   {
      FormedMessage = ":"+MyMessageFile()+":"+TheMessage+":\n";

      FileName = MessageFileName(PidToSendTo);
      FilePointer = open(FileName, O_WRONLY | O_APPEND); //| O_CREAT);
      write(FilePointer,(const char *)FormedMessage,FormedMessage.length());
      close(FilePointer);         
   }  
   return(SucessfullySend);
}





















