/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/*=============================================================================
   ToDo:
      - test speed of new VS malloc VS STMalloc      

=============================================================================*/

/*=============================================================================
   Note:
      - STMalloc uses STList to track memory leeks so STList can not use
        STMalloc to get memory.

=============================================================================*/

#include <STList.h>

STList::STList()
{
   FirstCell=(NodePointer *)malloc(sizeof(NodePointer));
   FirstCell->PointerToNextCell = NULL;
   FirstCell->PointerToPreviousCell = NULL;
   FirstCell->Item = NULL;
   LastCell = FirstCell;
   CurrentCell = FirstCell;
   NodeCount=0;
}

STList::~STList()
{
   if(NodeCount != 0)
   {
      printf("Warrning: STList::~STList() List is not empty! You may be causing a memory leak!\n");   
   }
   Clear();
   FirstCell->Item = NULL;
   //if(FirstCell->Item!=NULL)
   //{
   	// I don't think I should be freeing the Item pointer, what ever called the
      // STList should be allocating and freeing items for it self because we don't
      // know the size of the space to allocate in this scope.
      //free(FirstCell->Item);
   //}
   free(FirstCell);
}

void STList::Add(void *NewItemToAdd)
{
   NodePointer *NewCell;

   if(FirstCell->PointerToNextCell == NULL)
   {
      CurrentCell=FirstCell;
      NodeCount=0;
   } else
   {
      CurrentCell=LastCell;
   }
   CurrentCell->Item=NewItemToAdd;
   NewCell=(NodePointer *)malloc(sizeof(NodePointer));
   if(NewCell != NULL)
   {
      NewCell->Item = NULL;
      NewCell->PointerToNextCell = NULL;
      NewCell->PointerToPreviousCell = CurrentCell;
      CurrentCell->PointerToNextCell = NewCell;
      CurrentCell = NewCell;
      LastCell = NewCell;
      NodeCount++;
   }
}

unsigned int STList::Count()
{
   return(NodeCount);
}

void STList::Delete(int CellToKill)
{
   MoveToCell(CellToKill);
   if(CurrentCell != NULL)
   {
      PreviousCell=CurrentCell->PointerToPreviousCell;
      if(PreviousCell != NULL)
      {
         PreviousCell->PointerToNextCell=CurrentCell->PointerToNextCell;
      } else
      {
         FirstCell=CurrentCell->PointerToNextCell;
      }
      NextCell=CurrentCell->PointerToNextCell;
      if(NextCell != NULL)
      {
         NextCell->PointerToPreviousCell=PreviousCell;
      } else
      {
         LastCell=PreviousCell;
      }
		CurrentCell->Item = NULL;
      free(CurrentCell);
      CurrentCell = PreviousCell;
      NodeCount--;
   }
}

void STList::Clear()
{
   NodePointer *KillCell, *NextKillCell;
   
   if(NodeCount != 0)
   {
      printf("Warrning: STList::Clear() Clearing a list is not recomended because your items may remain.\n"); 
      printf("  NodeCount = %d\n",NodeCount);
      printf("\tThe recomended method is that you free your items and delete the list cell one at a time.\n");   
      printf("\texample...\n");   
      printf("\t\tfor(ALoopCounter = 0;ALoopCounter < YourList->Count();ALoopCounter++)\n");   
      printf("\t\t{\n");   
      printf("\t\t	YourPointer = (YourPointerType *)YourList->Item(0);\n");   
      printf("\t\t	free(YourPointer);\n");   
      printf("\t\t	YourList->Delete(0);\n");   
      printf("\t\t}\n");   
   }

	if(FirstCell->Item != NULL)
	{
		KillCell=FirstCell;
      NextKillCell=KillCell->PointerToNextCell;
 		// I don't think I should be freeing the Item pointer, what ever called the
    	// STList should be allocating and freeing items for it self because we don't
    	// know the size of the space to allocate in this scope.
  		//free(FirstCell->Item);
      FirstCell->Item = NULL;
  		free(FirstCell);
  		KillCell=NextKillCell;
  		if(KillCell != NULL)
  		{
     		while(KillCell->PointerToNextCell != NULL)
     		{
        		NextKillCell=KillCell->PointerToNextCell;
 				// I don't think I should be freeing the Item pointer, what ever called the
    			// STList should be allocating and freeing items for it self because we don't
    			// know the size of the space to allocate in this scope.
        		//if(KillCell->Item!=NULL)
        		//{
           	//	free(KillCell->Item);
        		//}
            KillCell->Item = NULL;
        		free(KillCell);
        		KillCell=NextKillCell;
     		}
         // I don't think I should be freeing the Item pointer, what ever called the
    		// STList should be allocating and freeing items for it self because we don't
    		// know the size of the space to allocate in this scope.
        	//if(KillCell->Item!=NULL)
        	//{
         //	free(KillCell->Item);
        	//}
         KillCell->Item = NULL;
     		free(KillCell);
  		}
  		FirstCell = (NodePointer *)malloc(sizeof(NodePointer));
  		FirstCell->PointerToNextCell = NULL;
  		FirstCell->PointerToPreviousCell = NULL;
  		FirstCell->Item = NULL;
      LastCell = FirstCell;
      CurrentCell = FirstCell;
      NodeCount = 0;
   }
}

void *STList::Item(int ItemIndex)
{
   void *TempReturn;
   
   TempReturn = NULL;
   MoveToCell(ItemIndex);
   if(CurrentCell != NULL)
   {
      TempReturn = CurrentCell->Item;
   }
   return(TempReturn);
}

void STList::MoveToCell(int CellToGoto)
{
   int FindCounter;
   
   FindCounter = 0;
   CurrentCell = FirstCell;
   
   while(CurrentCell->PointerToNextCell != NULL)
   {
      if(FindCounter == CellToGoto)
      {
         break;
      } else
      {
         CurrentCell = CurrentCell->PointerToNextCell;
         if(CurrentCell == NULL)
         {
            printf("Error: STList::MoveToCell(%d) Attempting to move to non-existint cell!\n",CellToGoto);
         }
         FindCounter++;
      }      
   }
   if(CurrentCell->PointerToNextCell != NULL)
   {
      if(FindCounter != CellToGoto)
      {
         printf("Warrning: STList::MoveToCell(%d) Could not move to cell beyond range!\n",CellToGoto);
         CurrentCell = NULL;
      }
   }
}

void STList::MoveTo(int SourceIndex, int DestIndex)
{
   printf("STList::MoveTo, Not implemented yet!\n");
}





















