#include "STPortDialog.h"

STPortDialog::STPortDialog(QWidget *parent, const char *name)
: STPortDialogGUI(parent, name)
{
   QString IniFileName;
   IniFileName = "PortSettings.ini";

   SuperIniSettings = new STSuperIni(IniFileName);

   DebugLevel=0;
   SetPortSelectValues();
   SetPortSpeedValue();
   SetDataBitsValue();
   SetParitySelectValue();
   SetStopBitsValue();
   LoadLastPortSettings();
   
   // Connect Signals to Slots
   connect(PortOnOff, SIGNAL(clicked()), SLOT(OnPortOnOffClick())); 
   connect(PortSelect,SIGNAL(activated(int)), SLOT(OnPortSettingsChanged())); 
   connect(PortSpeed,SIGNAL(activated(int)), SLOT(OnPortSettingsChanged())); 
   connect(DataBits,SIGNAL(activated(int)), SLOT(OnPortSettingsChanged())); 
   connect(ParitySelect,SIGNAL(activated(int)), SLOT(OnPortSettingsChanged())); 
   connect(StopBits,SIGNAL(activated(int)), SLOT(OnPortSettingsChanged()));    
}

int STPortDialog::GetBaud()
{
   QString CompairString;
   int TempReturnValue;
   
   CompairString = PortSpeed->currentText();
   // This is ugly! I tried to use a switch but it does not
   // work with QString's
   if(CompairString == "0 Baud")		TempReturnValue = 0; 
   if(CompairString == "50 Baud")	TempReturnValue = 50;
   if(CompairString == "75 Baud")	TempReturnValue = 75;
   if(CompairString == "110 Baud")	TempReturnValue = 110;
   if(CompairString == "134 Baud")	TempReturnValue = 134;
   if(CompairString == "150 Baud")	TempReturnValue = 150;
   if(CompairString == "200 Baud")	TempReturnValue = 200;
   if(CompairString == "300 Baud")	TempReturnValue = 300;
   if(CompairString == "600 Baud")	TempReturnValue = 600;
   if(CompairString == "1200 Baud")	TempReturnValue = 1200;
   if(CompairString == "1800 Baud")	TempReturnValue = 1800;
   if(CompairString == "2400 Baud")	TempReturnValue = 2400;
   if(CompairString == "4800 Baud")	TempReturnValue = 4800;
   if(CompairString == "9600 Baud")	TempReturnValue = 9600;
   if(CompairString == "19200 Baud")	TempReturnValue = 19200;
   if(CompairString == "38400 Baud")	TempReturnValue = 38400;
   if(CompairString == "57600 Baud")	TempReturnValue = 57600;
   if(CompairString == "115200 Baud")	TempReturnValue = 115200;
   if(CompairString == "230400 Baud")	TempReturnValue = 230400;
   
   return(TempReturnValue);
}

int STPortDialog::GetParity()
{
   QString CompairString;
   int TempReturnValue;
   
   CompairString = ParitySelect->currentText();
   // This is ugly! I tried to use a switch but it does not
   // work with QString's
   if(CompairString == "N")	TempReturnValue = NO_PARITY; 
   if(CompairString == "E")	TempReturnValue = EVEN_PARITY;
   if(CompairString == "O")	TempReturnValue = ODD_PARITY;
   
   return(TempReturnValue);
}

int STPortDialog::GetDataBits()
{
   QString CompairString;
   int TempReturnValue;
   
   CompairString = DataBits->currentText();
   // This is ugly! I tried to use a switch but it does not
   // work with QString's
   if(CompairString == "7")	TempReturnValue = WORD_LENGTH_7; 
   if(CompairString == "8")	TempReturnValue = WORD_LENGTH_8;
   
   return(TempReturnValue);
}

int STPortDialog::GetStopBits()
{
   QString CompairString;
   int TempReturnValue;
   
   CompairString = StopBits->currentText();
   // This is ugly! I tried to use a switch but it does not
   // work with QString's
   if(CompairString == "1")	TempReturnValue = STOP_BITS_1; 
   if(CompairString == "2")	TempReturnValue = STOP_BITS_2;

   return(TempReturnValue);
}

QString STPortDialog::GetPort()
{
   ReturnSting = PortSelect->currentText();
   return(ReturnSting);
}

void STPortDialog::SetActive(bool ActiveState)
{
   if(PortIsActive != ActiveState)
   {
      PortIsActive = ActiveState;
      if(ActiveState == false)
      {
         PortOnOffLabel->setText("Port is OFF");
      } else
      {
         PortOnOffLabel->setText("Port is ON");
      }
      emit ActiveChange(ActiveState);
   }
}

bool STPortDialog::SetDebugLevel(int NewDebugLevel)
{
   if(DebugLevel != NewDebugLevel)
   {
      DebugLevel = NewDebugLevel;
      printf("STPortDialog::SetDebugLevel = %d\n",DebugLevel);
      SuperIniSettings->SetDebugLevel(DebugLevel-1);
   }
   return(true);
}

void STPortDialog::LoadLastPortSettings()
{
   // I used these QStrings to stop warnings that qt generates
   QString TempValue;
   QString ValueName;
   QString CompairString;
   int ALoopCounter;
      
   PortSettingsChanged = true;
   SetActive(false);

   // Load old settings from INI file.
   ValueName = "PORT";
   TempValue = SuperIniSettings->GetValue(ValueName);
   for(ALoopCounter=0;ALoopCounter<PortSelect->count();ALoopCounter++)
   {
      CompairString = PortSelect->text(ALoopCounter);
      if(TempValue == CompairString)
      {
         PortSelect->setCurrentItem(ALoopCounter);
      }
   }
   
   ValueName = "SPEED";
   TempValue = SuperIniSettings->GetValue(ValueName);
   for(ALoopCounter=0;ALoopCounter<PortSpeed->count();ALoopCounter++)
   {
      CompairString = PortSpeed->text(ALoopCounter);
      if(TempValue == CompairString)
      {
         PortSpeed->setCurrentItem(ALoopCounter);
      }
   }
   
   ValueName = "DATA BITS";
   TempValue = SuperIniSettings->GetValue(ValueName);
   for(ALoopCounter=0;ALoopCounter<DataBits->count();ALoopCounter++)
   {
      CompairString = DataBits->text(ALoopCounter);
      if(TempValue == CompairString)
      {
         DataBits->setCurrentItem(ALoopCounter);
      }
   }
   
   ValueName = "PARITY";
   TempValue = SuperIniSettings->GetValue(ValueName);
   for(ALoopCounter=0;ALoopCounter<ParitySelect->count();ALoopCounter++)
   {
      CompairString = ParitySelect->text(ALoopCounter);
      if(TempValue == CompairString)
      {
         ParitySelect->setCurrentItem(ALoopCounter);
      }
   }
   
   ValueName = "STOP BITS";
   TempValue = SuperIniSettings->GetValue(ValueName);
   for(ALoopCounter=0;ALoopCounter<StopBits->count();ALoopCounter++)
   {
      CompairString = StopBits->text(ALoopCounter);
      if(TempValue == CompairString)
      {
         StopBits->setCurrentItem(ALoopCounter);
      }
   }   
}

void STPortDialog::OnPortSettingsChanged()
{
   // I used these QStrings to stop warnings that qt generates
   QString TempValue;
   QString ValueName;
      
   PortSettingsChanged = true;

   // Save new settings to INI file.
   ValueName = "PORT";
   TempValue = PortSelect->currentText();
   SuperIniSettings->SetValue(ValueName,TempValue);

   ValueName = "SPEED";
   TempValue = PortSpeed->currentText();
   SuperIniSettings->SetValue(ValueName,TempValue);
   
   ValueName = "DATA BITS";
   TempValue = DataBits->currentText();
   SuperIniSettings->SetValue(ValueName,TempValue);
   
   ValueName = "PARITY";
   TempValue = ParitySelect->currentText();
   SuperIniSettings->SetValue(ValueName,TempValue);
   
   ValueName = "STOP BITS";
   TempValue = StopBits->currentText();
   SuperIniSettings->SetValue(ValueName,TempValue);

   SetActive(false);
}

void STPortDialog::OnPortOnOffClick()
{
   SetActive(!PortIsActive);
}

void STPortDialog::SetDataBitsValue()
{
   DataBits->insertItem("7");
   DataBits->insertItem("8");
   DataBits->setCurrentItem(1);
}

void STPortDialog::SetParitySelectValue()
{
   ParitySelect->insertItem("N");
   ParitySelect->insertItem("E");
   ParitySelect->insertItem("O");   
   ParitySelect->setCurrentItem(0);
}

void STPortDialog::SetStopBitsValue()
{
   StopBits->insertItem("1");
   //StopBits->insertItem("1.5");
   StopBits->insertItem("2");
   StopBits->setCurrentItem(0);
}

void STPortDialog::SetPortSpeedValue()
{
   PortSpeed->insertItem("0 Baud");
   PortSpeed->insertItem("50 Baud");
   PortSpeed->insertItem("75 Baud");
   PortSpeed->insertItem("110 Baud");
   PortSpeed->insertItem("134 Baud");
   PortSpeed->insertItem("150 Baud");
   PortSpeed->insertItem("200 Baud");
   PortSpeed->insertItem("300 Baud");
   PortSpeed->insertItem("600 Baud");
   PortSpeed->insertItem("1200 Baud");
   PortSpeed->insertItem("1800 Baud");
   PortSpeed->insertItem("2400 Baud");
   PortSpeed->insertItem("4800 Baud");
   PortSpeed->insertItem("9600 Baud");
   PortSpeed->insertItem("19200 Baud");
   PortSpeed->insertItem("38400 Baud");
   PortSpeed->insertItem("57600 Baud");
   PortSpeed->insertItem("115200 Baud");
   PortSpeed->insertItem("230400 Baud");
   PortSpeed->setCurrentItem(0);
}

void STPortDialog::SetPortSelectValues()
{
   PortSelect->insertItem("ttyS0");
   PortSelect->insertItem("ttyS1");
   PortSelect->insertItem("ttyS2");
   PortSelect->insertItem("ttyS3");
   PortSelect->setCurrentItem(0);
}

STPortDialog::~STPortDialog()
{
   delete SuperIniSettings;
}
