/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

/*=============================================================================
   ToDo:

=============================================================================*/

#include <STStringList.h>

STStringList::STStringList()
: STList(), STObjectController()
{
   DiskIOBufferSize = 8192;
   FractalAddSavingOn = false;
}

STStringList::~STStringList()
{
   STStringList::Clear();
   if(FractalAddSavingOn == true)
   {
      DoneFractalAddSaving();
   }
}

void STStringList::StartFractalAddSaving(QString &FilePathAndName)
{
   if(FractalAddSavingOn == false)
   {
      FractalAddSavingOn = true;
      FractalSaveFileName = FilePathAndName;   
      FractalFileIOBuffer = (char *)malloc(DiskIOBufferSize);      
      WipeFile(FractalSaveFileName);
      
      FractalFilePointer = open(FractalSaveFileName, O_WRONLY|O_CREAT,S_IRWXU|S_IRWXG|S_IRWXO);
   } else
   {
      printf("ERROR: STStringList::SetFractalAddSaving\n");
      printf("       Attempting to activte twice!\n");
   }
}

void STStringList::StartFractalAddSaving(const char *FilePathAndName)
{
   QString TempString;

   TempString = FilePathAndName;
   StartFractalAddSaving(TempString);
}

void STStringList::DoneFractalAddSaving()
{
   if(FractalAddSavingOn == true)
   {
      FractalAddSavingOn = false;
      FractalSaveToFile();
      free(FractalFileIOBuffer);
      close(FractalFilePointer); 
   } else
   {
      printf("ERROR: STStringList::SetFractalAddSaving\n");
      printf("       Attempting to activte twice!\n");   
   }
}

QString STStringList::Item(int Index)
{
   char *NewStringPointer;
   
   if(DebugLevel > 2)
   {
      printf("STStringList::Item(%d)\n",Index);
   }

   NewStringPointer = (char *)STList::Item(Index);
   ReturnString = NewStringPointer;
   return(ReturnString);
}

unsigned int STStringList::Remove(int IndexToRemove)
{
   return(STStringList::Delete(IndexToRemove));
}

unsigned int STStringList::Delete(int IndexToRemove)
{
   char *NewStringPointer;
   
   if(DebugLevel > 2)
   {
      printf("STStringList::Delete(%d)\n",IndexToRemove);
   }

   NewStringPointer = (char *)STList::Item(IndexToRemove);
   
   free(NewStringPointer);
   STList::Delete(IndexToRemove);
   return(Count());
}

void STStringList::LoadFromFile(const char *FilePathAndName)
{
   QString FileName;
   
   FileName = FilePathAndName;
   LoadFromFile(FileName);
}

void STStringList::LoadFromFile(QString &FilePathAndName)
{
   char *FileIOBuffer;
   int DataInBufferSize,ALoopCounter;
   //FILE *FilePointer;
   unsigned int FilePointer;
   char ReadChar;
   int ReadCharIndex;
   char RealyBigString[1024];
   QString LoadFileName;
   STStringsEx StringsEx1;
   QString SwapString;

   FileIOBuffer = (char *)malloc(DiskIOBufferSize);
   STStringList::Clear();
	LoadFileName=FilePathAndName;
   StringsEx1.stripChars(LoadFileName,'"');
   
   FilePointer = open(LoadFileName, O_RDONLY);
   
   if(FilePointer!=0)
   {
      SwapString = "";
      ReadCharIndex = 0;
      RealyBigString[ReadCharIndex]=0;
      DataInBufferSize = read(FilePointer,FileIOBuffer,DiskIOBufferSize);      
      while(DataInBufferSize > 0)
      {
         for(ALoopCounter=0;ALoopCounter<DataInBufferSize;ALoopCounter++)
         {
            ReadChar = FileIOBuffer[ALoopCounter];
            if((ReadChar>31)||(ReadChar=='\t'))
            {
               RealyBigString[ReadCharIndex]=ReadChar;
               RealyBigString[ReadCharIndex+1]=0;
               ReadCharIndex++;                           
               if(ReadCharIndex > 1024)
               {
                  SwapString += RealyBigString;
                  ReadCharIndex = 0;
                  RealyBigString[ReadCharIndex]=0;
               }
            } else
            if(ReadChar == '\n')
            {
               SwapString += RealyBigString;
               ReadCharIndex = 0;
               RealyBigString[ReadCharIndex]=0;
               //printf("%s\n",(const char *)SwapString);
               Add(SwapString);
               SwapString = "";
            }  
         }
         DataInBufferSize = read(FilePointer,FileIOBuffer,DiskIOBufferSize);      
      }
      if(ReadCharIndex > 0)
      {
         SwapString += RealyBigString;
         ReadCharIndex = 0;
         RealyBigString[ReadCharIndex]=0;
         Add(SwapString);
         //printf("%s",(const char *)SwapString);
         SwapString = "";      
      }      
      //fclose(FilePointer);
      close(FilePointer);  
   }   
   free(FileIOBuffer);
}

unsigned int STStringList::Replace(QString &NewString, int Index)
{
   printf("Error:STStringList::Replace This function is not writen yet!\n");
   //Remove(Index);
   //insert(at(Index),NewString);
   
   return(0);
}

unsigned int STStringList::Add(const char *NewStringToAdd)
{
   QString TempString;
   int TempReturn;
   
   TempString = NewStringToAdd;
   TempReturn = Add(TempString);
   return(TempReturn);
}

unsigned int STStringList::Add(QString &NewStringToAdd)
{
   int TempReturn;
   char *NewStringPointer;
   int LengthOfString;
   
   if(DebugLevel > 2)
   {
      printf("STStringList::Add(%s)\n",(const char *)NewStringToAdd);
   }

   LengthOfString = NewStringToAdd.length();
   NewStringPointer = (char *)malloc(NewStringToAdd.length()+1);
   if(DebugLevel > 3)
   {
      printf("STStringList::Add(%s) @ 0x%08X -> 0x%08X\n",(const char *)NewStringToAdd,
         (uint)NewStringPointer,(uint)NewStringPointer + NewStringToAdd.length() + 1);
   }
   if(NewStringPointer > NULL)
   {
      strcpy(NewStringPointer,(const char *)NewStringToAdd);
      NewStringPointer[LengthOfString] = 0;
      STList::Add(NewStringPointer);
      //STList::Add((void *)NewStringPointer);
   } else
   {
      printf("List topped out at %d\n",Count());
   }
   
   TempReturn = Count();
   
   if(FractalAddSavingOn == true)
   {   
      FractalLineCount++;
      if(TempReturn > 100)
      {
         FractalSaveToFile();
      }
      TempReturn = FractalLineCount;      
   }

   return(TempReturn);
}

void STStringList::FractalSaveToFile()
{
   QString WriteString;
   int CutStringShort;
   int FileIOBufferIndex,LengthOfString;

   FileIOBufferIndex = 0;
   WriteString = "";
   LengthOfString = 0;
   while((Count())||(LengthOfString > 0))
   {      
      if(LengthOfString == 0)
      {
         WriteString = Item(0);
         STStringList::Remove(0);
      }
      LengthOfString = WriteString.length();
      if((FileIOBufferIndex + LengthOfString) < DiskIOBufferSize)
      {
         memcpy(FractalFileIOBuffer + FileIOBufferIndex,(const char *)WriteString,LengthOfString);
         FileIOBufferIndex += LengthOfString;
         if(FractalFileIOBuffer[FileIOBufferIndex - 1] != '\n')
         {
            FractalFileIOBuffer[FileIOBufferIndex] = '\n';
            FileIOBufferIndex++;
         }
         WriteString = "";
      } else
      {
         CutStringShort = DiskIOBufferSize - FileIOBufferIndex;
         memcpy(FractalFileIOBuffer + FileIOBufferIndex,(const char *)WriteString,CutStringShort);
         write(FractalFilePointer,FractalFileIOBuffer,DiskIOBufferSize);
         WriteString.remove(0,CutStringShort);
         FileIOBufferIndex = 0;      
      }
      LengthOfString = WriteString.length();
   }
   if(LengthOfString > 0)
   {
      memcpy(FractalFileIOBuffer + FileIOBufferIndex,(const char *)WriteString,LengthOfString);
      FileIOBufferIndex += LengthOfString;
      if(FractalFileIOBuffer[LengthOfString] != '\n')
      {
         LengthOfString++;
         FractalFileIOBuffer[LengthOfString] = '\n';
      } 
   }
   if(FileIOBufferIndex > 0)
   {
      write(FractalFilePointer,FractalFileIOBuffer,FileIOBufferIndex);
   }      
}

void STStringList::Clear()
{
   unsigned int ItemsToDelete,ALoopCounter;

   ItemsToDelete = Count();
   for(ALoopCounter=0;ALoopCounter<ItemsToDelete;ALoopCounter++)
   {
      STStringList::Remove(0);
   }
}

void STStringList::SaveToFile(const char *FilePathAndName)
{
   QString TheFileName;
   
   TheFileName = FilePathAndName;
   SaveToFile(TheFileName);
}

void STStringList::WipeFile(QString &WipePathAndFileName)
{
   FILE *EraseFilePointer;

   // I need to blank the file here. I did it in this goofy way because of two problems.
   // 1) Qt uses remove() to remove enteries from a list so I can not call remove to remove the file.
   // 2) open has no way to blank a file that exists, when writing to a file the old data in the
   //    file remains in it, that is not wanted. 
   EraseFilePointer = fopen(WipePathAndFileName,"w");
   fclose(EraseFilePointer);
}

void STStringList::SaveToFile(QString &FilePathAndName)
{
   unsigned int ALoopCounter;
   char *FileIOBuffer;
   unsigned int FilePointer;
   QString SaveFileName;
   STStringsEx StringsEx1;
   QString WriteString;
   int CutStringShort;
   int FileIOBufferIndex,LengthOfString;

   if(DebugLevel > 2)
   {
      printf("STStringList::SaveToFile(%s)\n",(const char *)FilePathAndName);
   }

	SaveFileName=FilePathAndName;
   StringsEx1.stripChars(SaveFileName,'"');

   FileIOBuffer = (char *)malloc(DiskIOBufferSize);
   
   WipeFile(SaveFileName);
   
	FilePointer = open(SaveFileName, O_WRONLY|O_CREAT,S_IRWXU|S_IRWXG|S_IRWXO);
   FileIOBufferIndex = 0;
   WriteString = "";
   ALoopCounter = 0;
   LengthOfString = 0;
   while((ALoopCounter < Count())||(LengthOfString > 0))
   {      
      if(LengthOfString == 0)
      {
         WriteString = Item(ALoopCounter);
      }
      LengthOfString = WriteString.length();
      if((FileIOBufferIndex + LengthOfString) < DiskIOBufferSize)
      {
         memcpy(FileIOBuffer + FileIOBufferIndex,(const char *)WriteString,LengthOfString);
         FileIOBufferIndex += LengthOfString;
         if(FileIOBuffer[FileIOBufferIndex - 1] != '\n')
         {
            FileIOBuffer[FileIOBufferIndex] = '\n';
            FileIOBufferIndex++;
         }
         WriteString = "";
         ALoopCounter++;
      } else
      {
         CutStringShort = DiskIOBufferSize - FileIOBufferIndex;
         memcpy(FileIOBuffer + FileIOBufferIndex,(const char *)WriteString,CutStringShort);
         write(FilePointer,FileIOBuffer,DiskIOBufferSize);
         WriteString.remove(0,CutStringShort);
         FileIOBufferIndex = 0;      
      }
      LengthOfString = WriteString.length();
   }
   if(LengthOfString > 0)
   {
      memcpy(FileIOBuffer + FileIOBufferIndex,(const char *)WriteString,LengthOfString);
      FileIOBufferIndex += LengthOfString;
      if(FileIOBuffer[LengthOfString] != '\n')
      {
         LengthOfString++;
         FileIOBuffer[LengthOfString] = '\n';
      } 
   }
   if(FileIOBufferIndex > 0)
   {
      write(FilePointer,FileIOBuffer,FileIOBufferIndex);
   }   
   close(FilePointer); 
   free(FileIOBuffer);
}























