/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

#include <STSuperIni.h>

bool STSuperIni::GetCheck(QString &ValueName)
{
   bool TempReturn;
   QString BoolValue;

   if(DebugLevel > 2)
   {
      printf("STSuperIni::GetCheck\n");
   }
   TempReturn = false;
   BoolValue = GetValue(ValueName);
   if(BoolValue.find("t",0,false) == 0)
   {
      TempReturn = true;
   }
   
   return(TempReturn);
}

bool STSuperIni::SetDebugLevel(int NewDebugLevel)
{
   if(DebugLevel != NewDebugLevel)
   {
      DebugLevel = NewDebugLevel;
      //PortDialog->SetDebugLevel(DebugLevel);
      printf("STSuperIni::SetDebugLevel = %d\n",DebugLevel);
   }
   return(true);
}

QString STSuperIni::GetValue(const char *ValueName)
{
   QString TempString;
   
   TempString = ValueName;
   RuturnString = GetValue(TempString);
   return(RuturnString);
}

QString STSuperIni::GetValue(QString &ValueName)
{
   unsigned int ALoopCounter;
   QString LocalValueName,FieldName;   
   STStringsEx StringsEx1;
   
   if(DebugLevel > 2)
   {
      printf("STSuperIni::GetValue from %s\n",(const char *)SifFileName);
   }
   RuturnString="";
   LocalValueName=ValueName+" = ";
   if(SifFileName.length()>0)
   {
      if(DebugLevel > 3)
      {
         printf("STSuperIni::SifFileName = %s\n",(const char *)SifFileName);
      }
      LoadFromFile(SifFileName);
      for(ALoopCounter = 0;ALoopCounter < Count();ALoopCounter++)
      {
         FieldName=Item(ALoopCounter);
         if(DebugLevel > 4)
         {
            printf("STSuperIni locating value in %s\n",(const char *)FieldName);
         }
         if(FieldName.find(LocalValueName,0,false)==0)
         {
            RuturnString=Item(ALoopCounter);
            RuturnString.remove(0,FieldName.find('=',1,false)+1);            
            RuturnString = StringsEx1.stripChars(RuturnString,' ');
            break;
         }
      }
   }
   if(DebugLevel > 4)
   {
      printf("%s = %s\n",(const char *)ValueName,(const char *)RuturnString);
   }
   return(RuturnString);
}

QString STSuperIni::GetValueDef(const char *ValueName,const char *DefaultValue)
{
   QString TempValueName, TempDefaultValue;
   
   TempValueName = ValueName;
   TempDefaultValue = DefaultValue;
   RuturnString = GetValueDef(TempValueName,TempDefaultValue);
   return(RuturnString);   
}

QString STSuperIni::GetValueDef(QString &ValueName,const char *DefaultValue)
{
   QString TempValueName, TempDefaultValue;
   
   TempValueName = ValueName;
   TempDefaultValue = DefaultValue;
   RuturnString = GetValueDef(TempValueName,TempDefaultValue);
   return(RuturnString);   
}

QString STSuperIni::GetValueDef(const char *ValueName,QString &DefaultValue)
{
   QString TempValueName, TempDefaultValue;
   
   TempValueName = ValueName;
   TempDefaultValue = DefaultValue;
   RuturnString = GetValueDef(TempValueName,TempDefaultValue);
   return(RuturnString);   
}

QString STSuperIni::GetValueDef(QString &ValueName,QString &DefaultValue)
{
   QString PosibileResult;
   //int SuspendSubCallDebug;
   
   if(DebugLevel > 2)
   {
      printf("STSuperIni::GetValueDef\n");
   }
   //SuspendSubCallDebug = DebugLevel;
   //DebugLevel = 0;
   PosibileResult=GetValue(ValueName);
   if(PosibileResult == "")
   {
      PosibileResult=DefaultValue;
   }
   RuturnString = PosibileResult;

   //DebugLevel = SuspendSubCallDebug;
   if(DebugLevel > 4)
   {
      printf("%s = %s\n",(const char *)ValueName,(const char *)RuturnString);
   }
   return(RuturnString);
}

void STSuperIni::SetCheck(QString &ValueName,bool Value)
{
   //printf("STSuperIni::SetCheck is not yet working!\n");
   QString TempSend;
   int SuspendSubCallDebug;
   
   if(DebugLevel > 2)
   {
      printf("STSuperIni::SetCheck\n");
   }
   SuspendSubCallDebug = DebugLevel;
   DebugLevel = 0;   
   if(Value == true)
   {
      TempSend = "True";
   } else
   {
      TempSend = "False";   
   }
   SetValue(ValueName,TempSend);

   DebugLevel = SuspendSubCallDebug;
   if(DebugLevel > 4)
   {
      printf("%s = %s\n",(const char *)ValueName,(const char *)TempSend);
   }
}

void STSuperIni::SetValue(const char *ValueName, const char *Value)
{
   QString TempString1,TempString2;
   
   TempString1 = ValueName;
   TempString2 = Value;
   SetValue(TempString1,TempString2);
}

void STSuperIni::SetValue(QString &ValueName, const char *Value)
{
   QString TempString1,TempString2;
   
   TempString1 = ValueName;
   TempString2 = Value;
   SetValue(TempString1,TempString2);
}

void STSuperIni::SetValue(const char *ValueName, QString &Value)
{
   QString TempString1,TempString2;
   
   TempString1 = ValueName;
   TempString2 = Value;
   SetValue(TempString1,TempString2);
}

void STSuperIni::SetValue(QString &ValueName, QString &Value)
{
   unsigned int ALoopCounter;
   QString LocalValueName,FieldName,LocalValue;
   STStringsEx StringsEx1;
   
   if(DebugLevel > 2)
   {
      printf("STSuperIni::SetValue\n");
   }

   LocalValue = StringsEx1.stripChars(Value,' ');
   LocalValueName=ValueName+" = ";
   if(SifFileName.length()>0)
   {
      LoadFromFile(SifFileName);
      for(ALoopCounter = 0;ALoopCounter < Count();ALoopCounter++)
      {
         FieldName=Item(ALoopCounter);
         if(FieldName.find(LocalValueName,0,false)==0)
         {
            Remove(ALoopCounter);
            ALoopCounter--;
         }
      }
      LocalValueName=ValueName+" = "+LocalValue;
      while(LocalValueName.find("\n",0,false) > -1)
      {
         if(DebugLevel > 4)
         {
            printf("Remove strange line feed.\n");
         }
         LocalValueName.remove(LocalValueName.find("\n",0,false),1);
      } 
      
      Add(LocalValueName);
      SaveToFile(SifFileName);   
   }
   if(DebugLevel > 4)
   {
      printf("%s = %s\n",(const char *)ValueName,(const char *)LocalValue);
   }
}

STSuperIni::STSuperIni(QString &PreSetFileName)
{
   InitSTSuperIni(PreSetFileName);
}

STSuperIni::STSuperIni(const char *PreSetFileName)
{
   QString QPreSetFileName;

   QPreSetFileName = PreSetFileName;
   InitSTSuperIni(QPreSetFileName);
}

void STSuperIni::InitSTSuperIni(QString &PreSetFileName)
{
   DebugLevel = 0;
   SifFileName=PreSetFileName;
}

STSuperIni::~STSuperIni()
{

}























