/*=============================================================================

    Copyright (C) 2001 Silicon Tao Technology Systems
    E-mail:  Support <support@silicontao.com>
    Web:     www.silicontao.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=============================================================================*/

#include <STXcom.h>

int STXcom::SetWordLength(int SelectedWordLength)
{
   if(DebugLevel > 2)
   {
      printf("STXcom::SetWordLength\n");
   }
   if(PortIsActive==false)
   {
	  WordLength=SelectedWordLength; 
   }
   return(0);
}

int STXcom::SetParity(int ParityValue)
{
   if(DebugLevel > 2)
   {
      printf("STXcom::SetParity\n");
   }
   if(PortIsActive==false)
   {
	  ParityBit=ParityValue;
   }
	return(0);
}

void STXcom::PrepareTheDialog()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::PrepareTheDialog\n");
   }
   if(PortDialog == NULL)
   {
      PortDialog = new STPortDialog(0,"");
      
      connect(PortDialog,SIGNAL(ActiveChange(bool)),SLOT(ChangeConnectionState(bool)));
   }
}

void STXcom::ChangeConnectionState(bool NewState)
{
   if(DebugLevel > 2)
   {
      printf("STXcom::ChangeConnectionState\n");
   }
   UserChangedStatusFromDialog = true;   
	if((PortIsActive == true) && (NewState == false))
	{
      Deactivate();
   } else
	if((PortIsActive == false) && (NewState == true))
   {
      Activate();
   }
}

bool STXcom::SetDebugLevel(int NewDebugLevel)
{
   if(DebugLevel != NewDebugLevel)
   {
      DebugLevel = NewDebugLevel;
      printf("STXcom::SetDebugLevel = %d\n",DebugLevel);
      if(PortDialog != NULL)
      {
         printf("PortDialog->SetDebugLevel\n");
         PortDialog->SetDebugLevel(DebugLevel-1);
      } else
      {
         printf("Not setting PortDialog->SetDebugLevel\n");
      }
   }
   return(true);
}

bool STXcom::IsDialogVisible()
{
   bool TempReturn;
   
   if(DebugLevel > 2)
   {
      printf("STXcom::IsDialogVisible\n");
   }
   TempReturn = false;
   if(PortDialog != NULL)
   {
      TempReturn = PortDialog->isVisible();
   }
   return(TempReturn);
}

void STXcom::ShowDialog()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::ShowDialog\n");
   }
   PrepareTheDialog();
   PortDialog->show();
}

void STXcom::HideDialog()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::HideDialog\n");
   }
   PrepareTheDialog();
   PortDialog->hide();
}

void STXcom::LoadSettings()
{
   QString TempString;
   bool ReturnToStatus;
   
   if(DebugLevel > 2)
   {
      printf("STXcom::LoadSettings\n");
   }
   ReturnToStatus = PortIsActive;
   if(ReturnToStatus == true)
   {
      Deactivate();
   }
   PrepareTheDialog();
   SetParity(PortDialog->GetParity());
	SetStopBits(PortDialog->GetStopBits());
	SetWordLength(PortDialog->GetDataBits());
   SetBaud(PortDialog->GetBaud());
   TempString = "/dev/"+PortDialog->GetPort();
   Port=TempString;
   if(ReturnToStatus == true)
   {
      Activate();
   }   
}

void STXcom::SaveSettings()
{
   PrepareTheDialog();

}

int STXcom::SetStopBits(int NumberOfStopBits)
{
   if(DebugLevel > 2)
   {
      printf("STXcom::SetStopBits\n");
   }
   if(PortIsActive==false)
   {
	  StopBits=NumberOfStopBits;
   }
	return(0);
}

STXcom::STXcom(QObject *parent, const char *name, QString &PortNumber)
:QObject(parent,name)
{
   InitSTXcom(PortNumber);
}

STXcom::STXcom(QObject *parent, const char *name, const char *PortNumber)
:QObject(parent,name)
{
   QString TempString;
   
   TempString = PortNumber;
   InitSTXcom(TempString);
}

void STXcom::InitSTXcom(QString &PortNumber)
{
   DebugLevel=0;
   BaudRate=B9600;
   PortIsActive=false;
   WordLength=WORD_LENGTH_8;
   StopBits=STOP_BITS_1;
   ParityBit=NO_PARITY;
   PortDialog = NULL;
   if(PortNumber == "from ini")
   {
      LoadSettings();
   } else
   {
      Port=PortNumber;
   }   
   FileDevice = 0;
}

STXcom::~STXcom()
{
   // Destructor       
   if(DebugLevel > 2)
   {
      printf("STXcom::~STXcom\n");
   }
   if(PortIsActive==true)
   {
      Deactivate();
   }
   if(PortDialog != NULL)
   {
      delete PortDialog;
   }
}

byte STXcom::Getb()
{
   byte Temp;
      
   if(DebugLevel > 5)
   {
      printf("STXcom::Getb\n");
   }
   Temp=(byte)Getc();   
   return(Temp);
}

void STXcom::StartTimeOutTimer(int TimeOutMax)
{
   TimeOutCount=0;
   LastTime=SystemTimer.second();
   MaxTimeCount=TimeOutMax;
}

bool STXcom::TimeIsUp()
{
   int ThisSecond;
   bool TempReturn;
   
   TempReturn=false;
   ThisSecond=SystemTimer.second();
   if(ThisSecond!=LastTime)
   {
      TimeOutCount++;
   }
   if(TimeOutCount>MaxTimeCount)
   {
      TempReturn=true;
   }
   return(TempReturn);
}

char STXcom::Getc()
{
	char Data;
   //int DataCount;

   if(DebugLevel > 5)
   {
      printf("STXcom::Getc\n");
   }
   Data=0;
   if(PortIsActive==true)
   {
      IsDataWaiting();
      if(RxDataIsWaiting==true)
      {
         Data=RxDataBuffer;
         RxDataBuffer=0;
         RxDataIsWaiting=false;      
      }
   }
	return(Data);
}

bool STXcom::IsDataWaiting()
{
   char ReadData;
   int DataCount;
   bool TempResult;

   TempResult=false;
   if(PortIsActive==true)
   {      
      if(RxDataIsWaiting==true)
      {
         TempResult=true;
      } else
      {         
         if(GetNumberOfBytesWaiting() > 0)
         {         
            DataCount=read(FileDevice,&ReadData,1);
         
            if(DataCount == 1) 
            {
               RxDataBuffer=ReadData;
               RxDataIsWaiting=true;
               TempResult=true;
            }
         }
      }
   }
	return(TempResult);
}

int STXcom::GetNumberOfBytesWaiting()
{
   int TempReturn;

   TempReturn = 0;
   if(PortIsActive==true)
   {      
      ioctl(FileDevice,FIONREAD,&TempReturn);
   }
   return(TempReturn);
}

int STXcom::SetBaud(int BaudCode)
{
   bool ValidCode;
   int ReturnErrorCode;
   
   if(DebugLevel > 2)
   {
      printf("STXcom::SetBaud\n");
   }
   ReturnErrorCode = ST_ERROR_NO_ERROR;   
   if(PortIsActive==false)
   {
      ValidCode = false;
      switch(BaudCode)
      {
         // All these listed values are valid.
         case 0:         
         case 50:        
         case 75:        
         case 110:       
         case 134:       
         case 150:       
         case 200:       
         case 300:       
         case 600:       
         case 1200:      
         case 1800:      
         case 2400:      
         case 4800:      
         case 9600:      
         case 19200:     
         case 38400:     
         case 57600:     
         case 115200:    
         case 230400:    
         case 460800:    ValidCode = true;  		break;
         default:        ValidCode = false;  	break;
      }
      if(ValidCode == true)
      { 
         BaudRate=BaudCode;
      } else
      {
         ReturnErrorCode = ST_ERROR_INVALID_BAUD_RATE;
         printf("ST_ERROR_INVALID_BAUD_RATE\n");
      }
   }
	return(ReturnErrorCode);
}

void STXcom::SetTransmitDelay(int MicroSeconds)
{
   if(DebugLevel > 2)
   {
      printf("STXcom::SetTransmitDelay(%d)\n",MicroSeconds);
   }      
   TransmitDelay = MicroSeconds;
}

int STXcom::PutBytes(byte *BytesSoragePointer,int WriteSize)
{
   int ALoopCounter;
   
   if(DebugLevel > 2)
   {
      printf("STXcom::PutBytes\n");
   }      

   for(ALoopCounter=0;ALoopCounter<WriteSize;ALoopCounter++)
   {
      if(ALoopCounter % 100 == 0)
      {
         emit DataReadWrite(ALoopCounter);
      }
      if(TransmitDelay > 0)
      {
         usleep(TransmitDelay);
      }
      if(DebugLevel > 2)
      {
         printf("0x%02X = %c\n",BytesSoragePointer[ALoopCounter],BytesSoragePointer[ALoopCounter]);
      }      
      Putc(BytesSoragePointer[ALoopCounter]);
   }
   return(0);
}

int STXcom::GetBytes(byte *BytesSoragePointer,int ReadSize)
{
   // should do a system time out here
   //int ALoopCounter,BLoopCounter,CLoopCounter,DataIndexCounter;
   int ReturnCount,TempByteCount,ByteCount;
   //bool Done;

   ReturnCount = 0;
      
   // RxDataIsWaiting == true only if data is waiting and we
   // previously called IsDataWaiting that read one byte and
   // stored it into RxDataBuffer. If we did that we must get
   // that byte first.
   if(RxDataIsWaiting == true)
   {
      ReadSize--;
      BytesSoragePointer[0]=RxDataBuffer;         
      RxDataBuffer=0;
      RxDataIsWaiting=false;      
      BytesSoragePointer++;
      ReturnCount = 1;
   }

   ByteCount = GetNumberOfBytesWaiting();
   if(ByteCount > ReadSize)
   {
      ByteCount = ReadSize;
   }

   if(ByteCount > 0)
   {
      TempByteCount = read(FileDevice,BytesSoragePointer,ByteCount);
      
      // It is posible that we only received one byte and it is in
      // RxDataBuffer, if so then ReturnCount is now -1, that is wrong.
      if(TempByteCount < 0)
      {
         TempByteCount = 0;
      }

      ReturnCount += TempByteCount;
   }  
   return(ReturnCount);   
}

QString STXcom::Gets()
{
   //ReturnString="";
   //return(ReturnString);
	return(0);
}

int STXcom::Putc(char Data)
{   
   if(DebugLevel > 5)
   {
      printf("STXcom::Putc\n");
   }

   if(PortIsActive==true)
   {
      write(FileDevice,&Data,1);
   }
	return(0); // Should be and error code
}

int STXcom::Puts(QString &String)
{
   if(DebugLevel > 5)
   {
      printf("STXcom::Puts\n");
   }
   printf("STXcom::Puts is not working yet\n");
	return(0);
}

QString STXcom::StopBitsAsText()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::StopBitsAsText\n");
   }
   switch(StopBits)
   {
      case STOP_BITS_1:		ReturnString="STOP_BITS_1";  	break;
      case STOP_BITS_2:		ReturnString="STOP_BITS_2"; 	break;    
      case STOP_BITS_15:   ReturnString="STOP_BITS_1.5";	break; 
   }   
   return(ReturnString);
}

QString STXcom::ParityAsText()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::ParityAsText\n");
   }
   switch(ParityBit)
   {
      case NO_PARITY:     ReturnString="NO_PARITY";  	break;
      case EVEN_PARITY:   ReturnString="EVEN_PARITY"; break;    
      case ODD_PARITY:    ReturnString="ODD_PARITY";	break; 
   }   
   return(ReturnString);
}

QString STXcom::WordLengthAsText()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::WordLengthAsText\n");
   }
   switch(WordLength)
   {
      case WORD_LENGTH_7:    ReturnString="WORD_LENGTH_7";	break;
      case WORD_LENGTH_8:    ReturnString="WORD_LENGTH_8";	break;
   }
   return(ReturnString);
}

QString STXcom::BaudRateAsText()
{
   if(DebugLevel > 2)
   {
      printf("STXcom::BaudRateAsText\n");
   }
   switch(BaudRate)
   {
      case 0:         ReturnString="B0";     	break;
      case 50:        ReturnString="B50";    	break;
      case 75:        ReturnString="B75";    	break;
      case 110:       ReturnString="B110";   	break;
      case 134:       ReturnString="B134";   	break;
      case 150:       ReturnString="B150";   	break;
      case 200:       ReturnString="B200";   	break;
      case 300:       ReturnString="B300";   	break;
      case 600:       ReturnString="B600";   	break;
      case 1200:      ReturnString="B1200";  	break;
      case 1800:      ReturnString="B1800";  	break;
      case 2400:      ReturnString="B2400";  	break;
      case 4800:      ReturnString="B4800";  	break;
      case 9600:      ReturnString="B9600";  	break;
      case 19200:     ReturnString="B19200"; 	break;
      case 38400:     ReturnString="B38400"; 	break;
      case 57600:     ReturnString="B57600"; 	break;
      case 115200:    ReturnString="B115200";	break;
      case 230400:    ReturnString="B230400";	break;
      case 460800:    ReturnString="B460800";	break;
      default:        ReturnString="B19200"; 	break;    
   }
   return(ReturnString);
}

bool STXcom::Activate()
{
   int ConfigurationBits;
   bool TempReturn, ReadyToActivate;
   
   if(DebugLevel > 1)
   {
      printf("STXcom::Activate\n");
   }
   
   ReadyToActivate=true;
   TempReturn=false;
   
   // I left this here so you are forced into sending it a 
   // /dev/ device path, that way you know you are connecting to
   // a device of some kind.
   if(Port.find("/dev/",0,false)!=0)
   {
      ReadyToActivate=false;
      printf("I can't start the port %s\n",(const char *)Port);
      printf("Make sure that you are accessing a /dev/ device!\n");
   }
	if(PortIsActive==true)
	{
      ReadyToActivate=false;
      printf("That port already active\n");
   }
   if(ReadyToActivate==true)
   {
      ConfigurationBits = (CLOCAL | CREAD);
      
      //if(HardWareFlowControl == true) ConfigurationBits |= CRTSCTS;
      
      switch(ParityBit)
      {
         case NO_PARITY:     ConfigurationBits &= ~(PARENB|PARODD);  break;
         case EVEN_PARITY:   ConfigurationBits |= PARENB;            break;    
         case ODD_PARITY:    ConfigurationBits |= (PARENB| PARODD);  break; 
      }   

      switch(WordLength)
      {
         case WORD_LENGTH_7:    ConfigurationBits|=CS7;     break;
         case WORD_LENGTH_8:    ConfigurationBits|=CS8;     break;
      }

      switch(BaudRate)
      {
         case 0:         ConfigurationBits|=B0;     	break;
         case 50:        ConfigurationBits|=B50;    	break;
         case 75:        ConfigurationBits|=B75;    	break;
         case 110:       ConfigurationBits|=B110;   	break;
         case 134:       ConfigurationBits|=B134;   	break;
         case 150:       ConfigurationBits|=B150;   	break;
         case 200:       ConfigurationBits|=B200;   	break;
         case 300:       ConfigurationBits|=B300;   	break;
         case 600:       ConfigurationBits|=B600;   	break;
         case 1200:      ConfigurationBits|=B1200;  	break;
         case 1800:      ConfigurationBits|=B1800;  	break;
         case 2400:      ConfigurationBits|=B2400;  	break;
         case 4800:      ConfigurationBits|=B4800;  	break;
         case 9600:      ConfigurationBits|=B9600;  	break;
         case 19200:     ConfigurationBits|=B19200; 	break;
         case 38400:     ConfigurationBits|=B38400; 	break;
         case 57600:     ConfigurationBits|=B57600; 	break;
         case 115200:    ConfigurationBits|=B115200;	break;
         case 230400:    ConfigurationBits|=B230400;	break;
         case 460800:    ConfigurationBits|=B460800;	break;
         default:        ConfigurationBits|=B19200; 	break;    
      }

      RxDataBuffer=0;
      RxDataIsWaiting=false;
	
      FileDevice = open(Port, O_RDWR | O_NOCTTY);
      if(FileDevice > 0)
      {       
         // save current port settings
         tcgetattr(FileDevice,&OldTerminalIOPointer); 
         bzero(&NewTerminalIOPointer,sizeof(NewTerminalIOPointer));
      
         // set new port settings 
         NewTerminalIOPointer.c_cflag = ConfigurationBits;
         NewTerminalIOPointer.c_iflag = IXANY; //IGNPAR | ICRNL;
         NewTerminalIOPointer.c_oflag = CR0;
         
         NewTerminalIOPointer.c_lflag = 0;
         
         // Wait for 0 bytes after 0 seconds, because we use this
         // we must also use the ioctl to find if data is waiting.
         NewTerminalIOPointer.c_cc[VMIN]=0;
         NewTerminalIOPointer.c_cc[VTIME]=0;            
      
         tcflush(FileDevice, TCIFLUSH);
         tcsetattr(FileDevice,TCSANOW,&NewTerminalIOPointer);
         PortIsActive=true;
         TempReturn=true;
         if(PortDialog != NULL)
         {
            PortDialog->SetActive(true);
         }
      }
   }
   return(TempReturn);
}

bool STXcom::IsPortActive()
{
   return(PortIsActive);
}

int STXcom::Deactivate()
{
	if(PortIsActive==true)
	{
      if(DebugLevel > 1)
      {
         printf("STXcom::Deactivate\n");
      }
      if(FileDevice > 0)
		{
         PortIsActive=false;
         tcsetattr(FileDevice,TCSANOW,&OldTerminalIOPointer);
         close(FileDevice);
         if(PortDialog != NULL)
         {
            PortDialog->SetActive(false);
         }
      }
	}
	return(0);
}


